#include <boost/filesystem/operations.hpp>
#include <boost/regex.hpp>
#include <iostream>
#include <fstream>
using namespace std;
namespace {

/////////////	
//regex stuff	
const char *ca_rxThrow = 
	"LIBTHROWABLE_THROW\\s*\\(\\s*(.*?)\\s*\\)";

const char *ca_rxThrowWithMsg = 
	"LIBTHROWABLE_THROW_WITH_MSG\\s*\\(\\s*(.*?)\\s*,\\s*(.*?)\\s*\\)";

const char *ca_fmtThrow = 
	"throw $1(LIBTHROWABLE_HERE)";

const char *ca_fmtThrowWithMsg = 
	"throw $1(LIBTHROWABLE_HERE, $2)";
////////////

/////////////////
//local functions
void showUsageAndExit();
void fileToString(const char *filename, string& contents);
void stringToFile(const char *filename, const string& contents);

void showUsageAndExit()
{
	cerr << "usage: lth_rename <filename>" << endl;
	exit(1);
}

void fileToString(const char *filename, string& contents)
{
	ifstream fStream(filename);
	char buf[256];
	while(fStream.good())
	{
		memset(buf, 0, 256);
		fStream.read(buf, 255);
		contents += buf;
	}
}

void stringToFile(const char *filename, const string& contents)
{
	ofstream fStream(filename);
	fStream << contents;
}
}//unnamed namespace

int main(int argc, char **argv)
{
	try
	{
		if(argc != 2)
			showUsageAndExit();
		if(!boost::filesystem::exists(argv[1]))
		{
			cerr << argv[1] << ": no such file" << endl;
			return 1;
		}
		if(boost::filesystem::is_directory(argv[1]))
		{
			cerr << argv[1] << ": not a file" << endl;
			return 1;
		}

		string rStr; fileToString(argv[1], rStr);
		string wStr;

		const boost::regex rxThrow(ca_rxThrow);
		const boost::regex rxThrowWithMsg(ca_rxThrowWithMsg);

		wStr = boost::regex_replace(rStr, rxThrow, ca_fmtThrow);
		wStr = boost::regex_replace(wStr, rxThrowWithMsg, ca_fmtThrowWithMsg);

		stringToFile(argv[1], wStr);
	}
	catch(const exception& e)
	{
		cerr << "unexpected failure:" << endl;
		cerr << "  what: " << e.what() << endl;
		return -1;
	}
	catch(...)
	{
		cerr << "unexpected failure:" << endl;
		cerr << "  unknown exception caught" << endl;
		return -1;
	}
	return 0;
}
