#include <iostream>
#include <libthrowable.h>
using namespace libthrowable;
using namespace std;
using libthrowable::util::stringify;
using libthrowable::util::addIndent;

namespace {
//define a new exception class	
namespace tags {
	struct SecuritySubsystemFailureTag
	{
		static const string& name()
		{
			static const string ret("SecuritySubsystemFailure");
			return ret;
		}
	};
}//namespace tags
typedef ThrowableType<Exception, tags::SecuritySubsystemFailureTag> SecuritySubsystemFailure;

namespace a_very_long {
namespace namespace_nested {

template<class T_NUM>	
string functionName(const T_NUM& arg1, const string& arg2, const string& arg3)
{
	LIBTHROWABLE_ASSERT(arg1 != 666);
	return string(stringify(arg1) + " " + arg2 + " " + arg3);
}

}}//namespaces
	
class BankAccount
{
	public:
	BankAccount(int sId) : _sId(sId), _access(false)	
	{ LIBTHROWABLE_TRACE_WITH_MSG(tp1, "constructing account with sId " + stringify(_sId)); }

	bool logIn(int sId)
	{
		LIBTHROWABLE_TRACE_WITH_MSG(tp1, "attempting to log in with sId " + stringify(sId));
		if(sId == _sId)
		{
			LIBTHROWABLE_TRACE_WITH_MSG(tp2, "access granted for sId " + stringify(_sId));
			_access = true;
			return true;
		}
		LIBTHROWABLE_TRACE_WITH_MSG(tp3, "access denied for sId " + stringify(_sId));
		_access = false;
		return false;
	}

	bool drawMoney(int sum)
	{
		LIBTHROWABLE_TRACE_WITH_MSG(tp1, "attemting to draw " + stringify(sum) + " from account " + stringify(_sId));
		if(!_access)
		{
			LIBTHROWABLE_TRACE_WITH_MSG(tp2, "not authorized; request rejected");
			return false;
		}
		try
		{
			return verifyId();
		}
		catch(const Throwable& th)
		{
			_access = false;
			string message =
				"the security subsystem has encountered a serious error ! "
				"For security reasons access has been reverted\n"
				"Security system response:\n"
				+ addIndent(th.what(), 2);
			 throw SecuritySubsystemFailure(LIBTHROWABLE_HERE,  message);

		}
	}

	bool logOut()
	{
		LIBTHROWABLE_TRACE_WITH_MSG(tp1, "logout for sId " + stringify(_sId));
		if(!_access)
		{
			LIBTHROWABLE_TRACE_WITH_MSG(tp2, "not loged in; ignoring");
			return false;
		}
		LIBTHROWABLE_TRACE_WITH_MSG(tp3, "ok");
		return true;
	}

	private:
	bool verifyId()
	{
		LIBTHROWABLE_TRACE(tp1);
		doDamnRecursiveStuff1();
		return true;
	}
	
	void doDamnRecursiveStuff1();
	void doDamnRecursiveStuff2();
	void doDamnRecursiveStuff3();

	int _sId;
	bool _access;
};

void BankAccount::doDamnRecursiveStuff1()
{
	LIBTHROWABLE_TRACE(tp1);
	static unsigned recCount = 0;
	++recCount;
	if(recCount == 4)
		a_very_long::namespace_nested::functionName(666, "argzett", "und weh");
	LIBTHROWABLE_TRACE_WITH_MSG(tp3, "starting new recursion");
	doDamnRecursiveStuff2();
}

void BankAccount::doDamnRecursiveStuff2()
{
	doDamnRecursiveStuff3();
	LIBTHROWABLE_TRACE(tp1);
}

void BankAccount::doDamnRecursiveStuff3()
{
	LIBTHROWABLE_TRACE(tp1);
	doDamnRecursiveStuff1();
	LIBTHROWABLE_TRACE(tp2);
}

}//unnamed namespace

int main()
{
	try
	{
		BankAccount acc(-1);
		acc.logIn(1 << 10);
		acc.logIn(-1);
		acc.drawMoney(100000);
	}
	catch(const Throwable& th)
	{
		cerr << th << endl;
	}
}
