#include <libthrowable.h>
#include <pthread.h>
#include <unistd.h>
#include <iostream>
using namespace libthrowable;
using namespace std;

namespace {
unsigned long int val1 = 0;
unsigned long int val2 = 0;

void* managerThread(void *p_val);
void* workerThread(void *p_val);

void* managerThread(void *p_val)
{
	LIBTHROWABLE_TRACE_WITH_MSG(tp1, "entering manager thread ...");
	try
	{
		pthread_t childThread;
		int rp = pthread_create(&childThread, NULL, workerThread, p_val);
		LIBTHROWABLE_ASSERT(rp == 0);
		sleep(1);
		rp = pthread_cancel(childThread);
		LIBTHROWABLE_ASSERT(rp == 0);
	}
	catch(const Throwable& th)
	{
		cerr << th << endl;
		abort();
	}
	LIBTHROWABLE_TRACE_WITH_MSG(tp2, "leaving manager thread ...");
	pthread_exit(NULL);
}

void* workerThread(void *p_val)
{
	LIBTHROWABLE_TRACE_WITH_MSG(tps, "entering worker thread ...");
	try
	{
		int rp = pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
		LIBTHROWABLE_ASSERT(rp == 0);
		rp = pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, NULL);
		LIBTHROWABLE_ASSERT(rp == 0);

		unsigned long int *p_long = static_cast<unsigned long int*>(p_val);

		LIBTHROWABLE_TRACE_WITH_MSG(tpc, "go ... ");
		while(true)
		{
			++(*p_long);
			if((*p_long) % 10000000 == 0)
				LIBTHROWABLE_TRACE_VAR(tp1, *p_long);
			pthread_testcancel();
		}
	}
	catch(const Throwable& th)
	{
		cerr << th << endl;
		abort();
	}
	pthread_exit(NULL);
}
}//local namespace
int main()
{
	try
	{
		pthread_t thread1, thread2;
		int rp1 = pthread_create(&thread1, NULL, managerThread, &val1);
		int rp2 = pthread_create(&thread2, NULL, managerThread, &val2);
		LIBTHROWABLE_ASSERT(!rp1 && !rp2);

		rp1 = pthread_join(thread1, NULL);
		rp2 = pthread_join(thread2, NULL);
		LIBTHROWABLE_ASSERT(!rp1 && !rp2);

		cout << "val1: " << val1 << endl;
		cout << "val2: " << val2 << endl;
		cout << "winner: " << ((val1 > val2) ? "thread1" : "thread2") << endl;
	}
	catch(const Throwable& th)
	{
		cout << th << endl;
		return -1;
	}
}
