#ifndef H_LIBTHROWABLE_INTERNAL
#define H_LIBTHROWABLE_INTERNAL
#include "libthrowable/system.h"
#include <stack>
#include <string>
namespace libthrowable {
namespace internal {

/** Optimizes strings to look nice for a given line width.
 * @note
 * 	This class is mostly here for internal use.
 * @warning
 * 	Don't use this class for now, as it is not yet sure
 * 	if this class will stay part of the API.
 * @see optimizeForLineWidth(const std::string&, int)	
 * @since libthrowable-0.9.1	
 * @warning for internal use only ! */
class LIBTHROWABLE_API StringAlignmentOptimizer
{
	public:
	/** Constructs a new StringAlignmentOptimizer.
	 * @param width
	 * 	The linewidth the string should be optimized for.
	 * 	If width < 40, no optimizing is done.*/
	StringAlignmentOptimizer(int width) throw();
	
	/** Optimizes str for a given linewidth.*/
	std::string optimize(const std::string& str) throw();

	private:
	//returns the indent of the current line;
	//-1 is returned if the line consists of spaces only or line is empty
	static long getIndent(const std::string& line) throw();
	//if line.size() > pos, this function tries to find a reasonable candidate before
	//pos, where a linebreak would be ok.
	static std::string::size_type getLinebreakPosNear(const std::string& line, std::string::size_type pos) throw();

	long getVirtualIndent(const std::string& line) throw();
	long getCurrentVirtualIndent() const throw()
	{ return _vIndentStack.empty() ? 0 : _vIndentStack.top(); }

	//optimizes the indendation of the lines in str.
	std::string adjustIndent(const std::string& str) throw();
	//adds additional linebreaks if this improves readability.
	std::string adjustLineBreaks(const std::string& str) const throw();

	const long _width;
	const long _maxIndent;
	std::stack<long> _vIndentStack;
};

/**
 * Aborts the execution of a program using libthrowable 
 * after a serious error occured inside the library.
 * @since libthrowable-0.9.4
 * @warning for internal use only !
 */
void LIBTHROWABLE_API LIBTHROWABLE_NORETURN emergencyAbort(const std::string& errmsg, const char* filename,
                    int line, const char* function, int errnum = 0) throw();
}}//namespaces
#endif

