/* libthrowable - a C++ library to ease error handling and debugging
 *
 * Copyright (C) Matthias Langer 2006 <mlangc@gmx.at>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef H_LIBTHROWABLE_ASSERT
#define H_LIBTHROWABLE_ASSERT

#ifndef LIBTHROWABLE_DISABLE_MACROS

#include <libthrowable/failed_assertion.h>
#include <libthrowable/macros/here.h>

#ifndef LIBTHROWABLE_NDEBUG
/** Assertion macro that throws a FailedAssertion object if the assertion fails.
 * @ingroup debugging
 * @note
 * 	Define LIBTHROWABLE_NDEBUG to remove the effects of this macro for production code.*/ 
#define LIBTHROWABLE_ASSERT(assertionArg) \
	do \
	{ \
		if(!(assertionArg)) \
		throw libthrowable::FailedAssertion(LIBTHROWABLE_HERE,  std::string("assertion \"" #assertionArg "\" failed !")); \
	}while(0)

#else
#define LIBTHROWABLE_ASSERT(assertionArg) \
	do { }while(0)

#endif //LIBTHROWABLE_NDEBUG
#endif //LIBTHROWABLE_DISABLE_MACROS
#endif //H_LIBTHROWABLE_ASSERT
