/* libthrowable - a C++ library to ease error handling and debugging
 *
 * Copyright (C) Matthias Langer 2006 <mlangc@gmx.at>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "user_fault.h"
#include "util/util.h"
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using std::string; using std::exception; using std::ostream;
using namespace libthrowable::util;
namespace libthrowable {
UserFault::UserFault(const std::string& msg) throw()
: Exception("UserFault:\n  message:\n" + addIndent(msg, 4), DontManipulateStr()), 
  _message(msg) {}

UserFault::UserFault(const char* filename, int linenumber, const std::string& msg) throw()
: Exception(internalStrFromFilenameLinenumberAndMessage("UserFault", filename, linenumber, msg), DontManipulateStr()),
  _message(msg) {}

UserFault::UserFault(const char *filename, int linenumber, const char *function, const std::string& msg) throw()
: Exception(internalStrFromFilenameLinenumberFunctionAndMessage("UserFault", filename, linenumber, function, msg), DontManipulateStr()),
  _message(msg) {}

const string& UserFault::getMessage() const throw()
{ return _message; }

UserFault::~UserFault() throw() {}

UserFault::UserFault(const string& msg, const string& passUnmodifiedToException) throw()
: Exception(passUnmodifiedToException, DontManipulateStr()), _message(msg) {}	

string& UserFault::getInternalMessage() throw()
{ return _message; }

ostream& operator<<(ostream& os, const UserFault& uf)
{
	os << uf.getMessage();
	return os;
}
}//namespace
