/* libthrowable - a C++ library to ease error handling and debugging
 *
 * Copyright (C) Matthias Langer 2006 <mlangc@gmx.at>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef H_LIBTHROWABLE_USER_FAULT_TYPE
#define H_LIBTHROWABLE_USER_FAULT_TYPE
#include "libthrowable/util/util.h"
#include "user_fault.h"
namespace libthrowable {
template<class UserFaultT, class TagT>
/** Derive new UserFaultTypes within seconds.
 * @ingroup exceptions
 * @see ThrowableType
 * @see UserFault*/
class UserFaultType : public UserFaultT
{
	public:
	explicit UserFaultType(const std::string& msg) throw()
	: UserFaultT(msg, TagT::name() + "\n  message:\n" + util::addIndent(msg, 4)) {}

	UserFaultType(const char *filename, int linenumber, const std::string& msg) throw()
	: UserFaultT(msg, UserFaultT::internalStrFromFilenameLinenumberAndMessage(TagT::name(), filename, linenumber, msg)) {}

	UserFaultType(const char *filename, int linenumber, const char *function, const std::string& msg) throw()
	: UserFaultT(msg, UserFaultT::internalStrFromFilenameLinenumberFunctionAndMessage(TagT::name(), filename, linenumber, function, msg)) {}
	
	virtual ~UserFaultType() throw() {}

	protected:
	UserFaultType(const std::string& msg, const std::string& passUnmodifiedToException) throw()
	: UserFaultT(msg, passUnmodifiedToException) {}
};
}//namespace
#endif
