/* libthrowable - a C++ library to ease error handling and debugging
 *
 * Copyright (C) Matthias Langer 2006 <mlangc@gmx.at>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "libthrowable/macros/here.h"
#include "util.h"
#include "libthrowable/internal/internal.h"
#include <sstream>
#include <vector>
#include <iostream>
#include <limits>
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif //HAVE_CONFIG_H
#ifdef HAVE_SYS_IOCTL_H
#include <sys/ioctl.h>
#endif //HAVE_SYS_IOCTL_H
#ifdef HAVE_TERMIOS_H
#include <termios.h>
#endif //HAVE_TERMIOS_H
#ifdef LIBTHROWABLE_WIN32
// if included before the standard
// C++ headers, the code will not
// compile. (at least this is the
// case with wineg++)
#include <windows.h>
// a macro called max might poison
// the code below in some implementations
#undef max
#endif
using namespace std;
namespace libthrowable {
namespace util {	
///////////
//functions
///////////
string addIndent(const string& str, unsigned indent) throw()
{
	string spaces(indent, ' ');
	string ret;
	istringstream hlp(str);

	unsigned counter=0;
	while(hlp.good())
	{
		if(counter != 0)
			ret += "\n";

		ret += spaces;
		string tmp;
		getline(hlp, tmp);
		ret += tmp;
		++counter;
	}
	return ret;
}

string
optimizeForLineWidth(const string& str, int width) throw()
{
	internal::StringAlignmentOptimizer optimizer(width);
	return optimizer.optimize(str);
}

int getTerminalWidth() throw()
{
	// termios.h is required for TIOCGWINSZ under Cygwin
	#if defined HAVE_SYS_IOCTL_H && (!defined LIBTHROWABLE_CYGWIN || defined HAVE_TERMIOS_H)
	struct winsize ws;
	if(ioctl(STDOUT_FILENO, TIOCGWINSZ, &ws) != -1)
		return ws.ws_col;
	#endif
	#ifdef LIBTHROWABLE_WIN32
	HANDLE handle = GetStdHandle(STD_OUTPUT_HANDLE);
	CONSOLE_SCREEN_BUFFER_INFO screenBufferInfo;
	if(GetConsoleScreenBufferInfo(handle, &screenBufferInfo) != 0)
		return screenBufferInfo.dwSize.X;
	#endif
	return 80;
}

}}//namespaces	
