/* libthrowable - a C++ library to ease error handling and debugging
 *
 * Copyright (C) Matthias Langer 2006 <mlangc@gmx.at>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef H_LIBTHROWABLE_UTIL
#define H_LIBTHROWABLE_UTIL
#include <string>
#include <sstream>
#include "libthrowable/system.h"
namespace libthrowable {
/** Utilities, mostly, but not only, for internal use.
 * @since libthrowable-0.9.1*/
namespace util {

/** Use this one to convert various objects to std::string.
 * @ingroup various
 * @param arg
 * 	If "os" is of type std::ostream, "os << arg;" must be a legal statement.
 * @since libthrowable-0.9.1 */ 	
template<class OStreamableT> 
std::string LIBTHROWABLE_API stringify(const OStreamableT& arg) throw();

/** Returns the width of the current terminal or 80.
 * @ingroup various
 * @see optimizeForTerminalWidth(const std::string&)
 * @since libthrowable-0.9.1*/
int LIBTHROWABLE_API getTerminalWidth() throw();

/** Helper function to add indent to strings.
 * @ingroup various
 * @since libthrowable-0.9.1*/
std::string LIBTHROWABLE_API addIndent(const std::string& str, unsigned indent) throw();

/** This function returns str optimized for a given linewidth.
 * @ingroup various
 * @see optimizeForTerminalWidth(const std::string&)
 * @since libthrowable-0.9.1*/
std::string LIBTHROWABLE_API optimizeForLineWidth(const std::string& str, int width) throw();

/** This function returns str optimized for the current terminal width.
 * @ingroup various
 * @see optimizeForLineWidth(const std::string&, int)
 * @see getTerminalWidth()
 * @since libthrowable-1.0.0.*/
inline std::string LIBTHROWABLE_API 
	optimizeForTerminalWidth(const std::string& str) throw()
{ return optimizeForLineWidth(str, getTerminalWidth()); }

template<class OStreamableT> 
std::string stringify(const OStreamableT& arg) throw()
{
	std::ostringstream os;
	os << arg;
	return os.str();
}

}}//namespaces
#endif

